@@ -89,19 +89,15 @@ class TestIMCPreprocessing:
     def test_create_panel_from_mcd_file(
         self, imc_test_data_steinbock_path: Path
     ):
-        imc.create_panel_from_mcd_file(
-            imc_test_data_steinbock_path
-            / "raw"
-            / "20210305_NE_mockData1"
-            / "20210305_NE_mockData1.mcd"
-        )  # TODO
+        mcd_files = imc.list_mcd_files(
+            imc_test_data_steinbock_path / "raw" / "20210305_NE_mockData1"
+        )
+        imc.create_panel_from_mcd_files(mcd_files)  # TODO
 
     def test_create_panel_from_txt_file(
         self, imc_test_data_steinbock_path: Path
     ):
-        imc.create_panel_from_txt_file(
-            imc_test_data_steinbock_path
-            / "raw"
-            / "20210305_NE_mockData1"
-            / "20210305_NE_mockData1_ROI_001_1.txt"
-        )  # TODO
+        txt_files = imc.list_txt_files(
+            imc_test_data_steinbock_path / "raw" / "20210305_NE_mockData1"
+        )
+        imc.create_panel_from_txt_files(txt_files)  # TODO
